==> utils-group-in-0.0.7.tgz: package/README.md <==
<!--

@license Apache-2.0

Copyright (c) 2018 The Stdlib Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-->

# groupIn

[![NPM version][npm-image]][npm-url] [![Build Status][test-image]][test-url] [![Coverage Status][coverage-image]][coverage-url] <!-- [![dependencies][dependencies-image]][dependencies-url] -->

> Group an object's **own** and **inherited** property values according to an indicator function.

<!-- Section to include introductory text. Make sure to keep an empty line after the intro `section` element and another before the `/section` close. -->

<section class="intro">

</section>

<!-- /.intro -->

<!-- Package usage documentation. -->

<section class="installation">

## Installation

```bash
npm install @stdlib/utils-group-in
```

</section>

<section class="usage">

## Usage

```javascript
var groupIn = require( '@stdlib/utils-group-in' );
```

#### groupIn( obj, \[options,] indicator )

Groups an object's **own** and **inherited** property values according to an `indicator` function, which specifies which group a value in the input `object` belongs to.

```javascript
function indicator( v ) {
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var out = groupIn( obj, indicator );
// e.g., returns { 'b': [ 'beep', 'boop', 'bar' ], 'f': [ 'foo' ] }
```

An `indicator` function is provided two arguments:

-   `value`: object value
-   `key`: object index

```javascript
function indicator( v, k ) {
    console.log( '%s: %s', k, v );
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var out = groupIn( obj, indicator );
// e.g., returns { 'b': [ 'beep', 'boop', 'bar' ], 'f': [ 'foo' ] }
```

The function accepts the following `options`:

-   `returns`: specifies the output format. If the option equals `'values'`, the function outputs values. If the option equals `'keys'`, the function outputs keys. If the option equals `'*'`, the function outputs both keys and values. Default: `'values'`.
-   `thisArg`: execution context.

By default, the function returns object values. To return object keys, set the `returns` option to `'keys'`.

```javascript
function indicator( v ) {
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var opts = {
    'returns': 'keys'
};
var out = groupIn( obj, opts, indicator );
// e.g., returns { 'b': [ 'a', 'b', 'd' ], 'f': [ 'c' ] }
```

To return key-value pairs, set the `returns` option to `'*'`.

```javascript
function indicator( v ) {
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var opts = {
    'returns': '*'
};
var out = groupIn( obj, opts, indicator );
// e.g., returns { 'b': [ [ 'a', 'beep' ], [ 'b', 'boop ], [ 'd', 'bar' ] ], 'f': [ [ 'c', 'foo' ] ] }
```

To set the `indicator` execution context, provide a `thisArg`.

```javascript
function indicator( v ) {
    this.count += 1;
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var context = {
    'count': 0
};
var opts = {
    'thisArg': context
};

var out = groupIn( obj, opts, indicator );
// e.g., returns { 'b': [ 'beep', 'boop', 'bar' ], 'f': [ 'foo' ] }

console.log( context.count );
// => 4
```

</section>

<!-- /.usage -->

<!-- Package usage notes. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="notes">

## Notes

-   Iteration order is **not** guaranteed, as `object` key enumeration is not specified according to the [ECMAScript specification][ecma-262-for-in]. In practice, however, most engines use insertion order to sort an `object`'s keys, thus allowing for deterministic iteration.

-   Because iteration order is **not** guaranteed, result order is **not** guaranteed.

-   The function determines the list of own **and** inherited enumerable properties **before** invoking the provided function. Hence, any modifications made to the input `object` **after** calling this function (such as adding and removing properties) will **not** affect the list of visited properties.

-   The value returned by an `indicator` function should be a value which can be serialized as an `object` key. As a counterexample,

    ```javascript
    function indicator( v ) {
        return {};
    }

    function Foo() {
        this.a = 'beep';
        this.b = 'boop';
        return this;
    }

    Foo.prototype = Object.create( null );
    Foo.prototype.c = 'foo';
    Foo.prototype.d = 'bar';

    var obj = new Foo();

    var out = groupIn( obj, indicator );
    // e.g., returns { '[object Object]': [ 'beep', 'boop', 'foo', 'bar' ] }
    ```

    while each group identifier is unique, all object values resolve to the same group because each group identifier serializes to the same `string`.

</section>

<!-- /.notes -->

<!-- Package usage examples. -->

<section class="examples">

## Examples

<!-- eslint no-undef: "error" -->

```javascript
var randu = require( '@stdlib/random-base-randu' );
var fromCodePoint = require( '@stdlib/string-from-code-point' );
var groupIn = require( '@stdlib/utils-group-in' );

var key;
var obj;
var out;
var i;

function Foo() {
    var key;
    var i;
    for ( i = 0; i < 50; i++ ) {
        key = fromCodePoint( 147+i );
        this[ key ] = randu();
    }
    return this;
}

Foo.prototype = Object.create( null );
for ( i = 0; i < 50; i++ ) {
    key = fromCodePoint( 97+i );
    Foo.prototype[ key ] = randu();
}

// Generate a random object:
obj = new Foo();

// Compute the groups...
function indicator( v ) {
    if ( v < 0.5 ) {
        return 'low';
    }
    return 'high';
}
out = groupIn( obj, indicator );
console.log( out );
```

</section>

<!-- /.examples -->

<!-- Section to include cited references. If references are included, add a horizontal rule *before* the section. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="references">

</section>

<!-- /.references -->

<!-- Section for related `stdlib` packages. Do not manually edit this section, as it is automatically populated. -->

<section class="related">

* * *

## See Also

-   <span class="package-name">[`@stdlib/utils/bifurcate-in`][@stdlib/utils/bifurcate-in]</span><span class="delimiter">: </span><span class="description">split an object's own and inherited property values into two groups according to a predicate function.</span>
-   <span class="package-name">[`@stdlib/utils/group-by`][@stdlib/utils/group-by]</span><span class="delimiter">: </span><span class="description">group values according to an indicator function.</span>
-   <span class="package-name">[`@stdlib/utils/group-own`][@stdlib/utils/group-own]</span><span class="delimiter">: </span><span class="description">group an object's own property values according to an indicator function.</span>

</section>

<!-- /.related -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->


<section class="main-repo" >

* * *

## Notice

This package is part of [stdlib][stdlib], a standard library for JavaScript and Node.js, with an emphasis on numerical and scientific computing. The library provides a collection of robust, high performance libraries for mathematics, statistics, streams, utilities, and more.

For more information on the project, filing bug reports and feature requests, and guidance on how to develop [stdlib][stdlib], see the main project [repository][stdlib].

#### Community

[![Chat][chat-image]][chat-url]

---

## License

See [LICENSE][stdlib-license].


## Copyright

Copyright &copy; 2016-2022. The Stdlib [Authors][stdlib-authors].

</section>

<!-- /.stdlib -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="links">

[npm-image]: http://img.shields.io/npm/v/@stdlib/utils-group-in.svg
[npm-url]: https://npmjs.org/package/@stdlib/utils-group-in

[test-image]: https://github.com/stdlib-js/utils-group-in/actions/workflows/test.yml/badge.svg
[test-url]: https://github.com/stdlib-js/utils-group-in/actions/workflows/test.yml

[coverage-image]: https://img.shields.io/codecov/c/github/stdlib-js/utils-group-in/main.svg
[coverage-url]: https://codecov.io/github/stdlib-js/utils-group-in?branch=main

<!--

[dependencies-image]: https://img.shields.io/david/stdlib-js/utils-group-in.svg
[dependencies-url]: https://david-dm.org/stdlib-js/utils-group-in/main

-->

[umd]: https://github.com/umdjs/umd
[es-module]: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules

[deno-url]: https://github.com/stdlib-js/utils-group-in/tree/deno
[umd-url]: https://github.com/stdlib-js/utils-group-in/tree/umd
[esm-url]: https://github.com/stdlib-js/utils-group-in/tree/esm

[chat-image]: https://img.shields.io/gitter/room/stdlib-js/stdlib.svg
[chat-url]: https://gitter.im/stdlib-js/stdlib/

[stdlib]: https://github.com/stdlib-js/stdlib

[stdlib-authors]: https://github.com/stdlib-js/stdlib/graphs/contributors

[stdlib-license]: https://raw.githubusercontent.com/stdlib-js/utils-group-in/main/LICENSE

[ecma-262-for-in]: http://www.ecma-international.org/ecma-262/5.1/#sec-12.6.4

<!-- <related-links> -->

[@stdlib/utils/bifurcate-in]: https://www.npmjs.com/package/@stdlib/utils-bifurcate-in

[@stdlib/utils/group-by]: https://www.npmjs.com/package/@stdlib/utils-group-by

[@stdlib/utils/group-own]: https://www.npmjs.com/package/@stdlib/utils-group-own

<!-- </related-links> -->

</section>

<!-- /.links -->

==> utils-group-in-0.1.0.tgz: package/README.md <==
<!--

@license Apache-2.0

Copyright (c) 2018 The Stdlib Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-->


<details>
  <summary>
    About stdlib...
  </summary>
  <p>We believe in a future in which the web is a preferred environment for numerical computation. To help realize this future, we've built stdlib. stdlib is a standard library, with an emphasis on numerical and scientific computation, written in JavaScript (and C) for execution in browsers and in Node.js.</p>
  <p>The library is fully decomposable, being architected in such a way that you can swap out and mix and match APIs and functionality to cater to your exact preferences and use cases.</p>
  <p>When you use stdlib, you can be absolutely certain that you are using the most thorough, rigorous, well-written, studied, documented, tested, measured, and high-quality code out there.</p>
  <p>To join us in bringing numerical computing to the web, get started by checking us out on <a href="https://github.com/stdlib-js/stdlib">GitHub</a>, and please consider <a href="https://opencollective.com/stdlib">financially supporting stdlib</a>. We greatly appreciate your continued support!</p>
</details>

# groupIn

[![NPM version][npm-image]][npm-url] [![Build Status][test-image]][test-url] [![Coverage Status][coverage-image]][coverage-url] <!-- [![dependencies][dependencies-image]][dependencies-url] -->

> Group an object's **own** and **inherited** property values according to an indicator function.

<!-- Section to include introductory text. Make sure to keep an empty line after the intro `section` element and another before the `/section` close. -->

<section class="intro">

</section>

<!-- /.intro -->

<!-- Package usage documentation. -->

<section class="installation">

## Installation

```bash
npm install @stdlib/utils-group-in
```

</section>

<section class="usage">

## Usage

```javascript
var groupIn = require( '@stdlib/utils-group-in' );
```

#### groupIn( obj, \[options,] indicator )

Groups an object's **own** and **inherited** property values according to an `indicator` function, which specifies which group a value in the input `object` belongs to.

```javascript
function indicator( v ) {
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var out = groupIn( obj, indicator );
// e.g., returns { 'b': [ 'beep', 'boop', 'bar' ], 'f': [ 'foo' ] }
```

An `indicator` function is provided two arguments:

-   `value`: object value
-   `key`: object index

```javascript
function indicator( v, k ) {
    console.log( '%s: %s', k, v );
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var out = groupIn( obj, indicator );
// e.g., returns { 'b': [ 'beep', 'boop', 'bar' ], 'f': [ 'foo' ] }
```

The function accepts the following `options`:

-   `returns`: specifies the output format. If the option equals `'values'`, the function outputs values. If the option equals `'keys'`, the function outputs keys. If the option equals `'*'`, the function outputs both keys and values. Default: `'values'`.
-   `thisArg`: execution context.

By default, the function returns object values. To return object keys, set the `returns` option to `'keys'`.

```javascript
function indicator( v ) {
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var opts = {
    'returns': 'keys'
};
var out = groupIn( obj, opts, indicator );
// e.g., returns { 'b': [ 'a', 'b', 'd' ], 'f': [ 'c' ] }
```

To return key-value pairs, set the `returns` option to `'*'`.

```javascript
function indicator( v ) {
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var opts = {
    'returns': '*'
};
var out = groupIn( obj, opts, indicator );
// e.g., returns { 'b': [ [ 'a', 'beep' ], [ 'b', 'boop ], [ 'd', 'bar' ] ], 'f': [ [ 'c', 'foo' ] ] }
```

To set the `indicator` execution context, provide a `thisArg`.

```javascript
function indicator( v ) {
    this.count += 1;
    return v[ 0 ];
}

function Foo() {
    this.a = 'beep';
    this.b = 'boop';
    return this;
}

Foo.prototype = Object.create( null );
Foo.prototype.c = 'foo';
Foo.prototype.d = 'bar';

var obj = new Foo();

var context = {
    'count': 0
};
var opts = {
    'thisArg': context
};

var out = groupIn( obj, opts, indicator );
// e.g., returns { 'b': [ 'beep', 'boop', 'bar' ], 'f': [ 'foo' ] }

console.log( context.count );
// => 4
```

</section>

<!-- /.usage -->

<!-- Package usage notes. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="notes">

## Notes

-   Iteration order is **not** guaranteed, as `object` key enumeration is not specified according to the [ECMAScript specification][ecma-262-for-in]. In practice, however, most engines use insertion order to sort an `object`'s keys, thus allowing for deterministic iteration.

-   Because iteration order is **not** guaranteed, result order is **not** guaranteed.

-   The function determines the list of own **and** inherited enumerable properties **before** invoking the provided function. Hence, any modifications made to the input `object` **after** calling this function (such as adding and removing properties) will **not** affect the list of visited properties.

-   The value returned by an `indicator` function should be a value which can be serialized as an `object` key. As a counterexample,

    ```javascript
    function indicator( v ) {
        return {};
    }

    function Foo() {
        this.a = 'beep';
        this.b = 'boop';
        return this;
    }

    Foo.prototype = Object.create( null );
    Foo.prototype.c = 'foo';
    Foo.prototype.d = 'bar';

    var obj = new Foo();

    var out = groupIn( obj, indicator );
    // e.g., returns { '[object Object]': [ 'beep', 'boop', 'foo', 'bar' ] }
    ```

    while each group identifier is unique, all object values resolve to the same group because each group identifier serializes to the same `string`.

</section>

<!-- /.notes -->

<!-- Package usage examples. -->

<section class="examples">

## Examples

<!-- eslint no-undef: "error" -->

```javascript
var randu = require( '@stdlib/random-base-randu' );
var fromCodePoint = require( '@stdlib/string-from-code-point' );
var groupIn = require( '@stdlib/utils-group-in' );

var key;
var obj;
var out;
var i;

function Foo() {
    var key;
    var i;
    for ( i = 0; i < 50; i++ ) {
        key = fromCodePoint( 147+i );
        this[ key ] = randu();
    }
    return this;
}

Foo.prototype = Object.create( null );
for ( i = 0; i < 50; i++ ) {
    key = fromCodePoint( 97+i );
    Foo.prototype[ key ] = randu();
}

// Generate a random object:
obj = new Foo();

// Compute the groups...
function indicator( v ) {
    if ( v < 0.5 ) {
        return 'low';
    }
    return 'high';
}
out = groupIn( obj, indicator );
console.log( out );
```

</section>

<!-- /.examples -->

<!-- Section to include cited references. If references are included, add a horizontal rule *before* the section. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="references">

</section>

<!-- /.references -->

<!-- Section for related `stdlib` packages. Do not manually edit this section, as it is automatically populated. -->

<section class="related">

* * *

## See Also

-   <span class="package-name">[`@stdlib/utils-bifurcate-in`][@stdlib/utils/bifurcate-in]</span><span class="delimiter">: </span><span class="description">split an object's own and inherited property values into two groups according to a predicate function.</span>
-   <span class="package-name">[`@stdlib/utils-group-by`][@stdlib/utils/group-by]</span><span class="delimiter">: </span><span class="description">group values according to an indicator function.</span>
-   <span class="package-name">[`@stdlib/utils-group-own`][@stdlib/utils/group-own]</span><span class="delimiter">: </span><span class="description">group an object's own property values according to an indicator function.</span>

</section>

<!-- /.related -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->


<section class="main-repo" >

* * *

## Notice

This package is part of [stdlib][stdlib], a standard library for JavaScript and Node.js, with an emphasis on numerical and scientific computing. The library provides a collection of robust, high performance libraries for mathematics, statistics, streams, utilities, and more.

For more information on the project, filing bug reports and feature requests, and guidance on how to develop [stdlib][stdlib], see the main project [repository][stdlib].

#### Community

[![Chat][chat-image]][chat-url]

---

## License

See [LICENSE][stdlib-license].


## Copyright

Copyright &copy; 2016-2023. The Stdlib [Authors][stdlib-authors].

</section>

<!-- /.stdlib -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="links">

[npm-image]: http://img.shields.io/npm/v/@stdlib/utils-group-in.svg
[npm-url]: https://npmjs.org/package/@stdlib/utils-group-in

[test-image]: https://github.com/stdlib-js/utils-group-in/actions/workflows/test.yml/badge.svg?branch=v0.1.0
[test-url]: https://github.com/stdlib-js/utils-group-in/actions/workflows/test.yml?query=branch:v0.1.0

[coverage-image]: https://img.shields.io/codecov/c/github/stdlib-js/utils-group-in/main.svg
[coverage-url]: https://codecov.io/github/stdlib-js/utils-group-in?branch=main

<!--

[dependencies-image]: https://img.shields.io/david/stdlib-js/utils-group-in.svg
[dependencies-url]: https://david-dm.org/stdlib-js/utils-group-in/main

-->

[chat-image]: https://img.shields.io/gitter/room/stdlib-js/stdlib.svg
[chat-url]: https://app.gitter.im/#/room/#stdlib-js_stdlib:gitter.im

[stdlib]: https://github.com/stdlib-js/stdlib

[stdlib-authors]: https://github.com/stdlib-js/stdlib/graphs/contributors

[umd]: https://github.com/umdjs/umd
[es-module]: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules

[deno-url]: https://github.com/stdlib-js/utils-group-in/tree/deno
[umd-url]: https://github.com/stdlib-js/utils-group-in/tree/umd
[esm-url]: https://github.com/stdlib-js/utils-group-in/tree/esm
[branches-url]: https://github.com/stdlib-js/utils-group-in/blob/main/branches.md

[stdlib-license]: https://raw.githubusercontent.com/stdlib-js/utils-group-in/main/LICENSE

[ecma-262-for-in]: https://262.ecma-international.org/5.1/#sec-12.6.4

<!-- <related-links> -->

[@stdlib/utils/bifurcate-in]: https://www.npmjs.com/package/@stdlib/utils-bifurcate-in

[@stdlib/utils/group-by]: https://www.npmjs.com/package/@stdlib/utils-group-by

[@stdlib/utils/group-own]: https://www.npmjs.com/package/@stdlib/utils-group-own

<!-- </related-links> -->

</section>

<!-- /.links -->
